'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createModel;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Types = require('./Types');

var _Types2 = _interopRequireDefault(_Types);

var _isModel = require('./isModel');

var _isModel2 = _interopRequireDefault(_isModel);

var _isCollection = require('./isCollection');

var _isCollection2 = _interopRequireDefault(_isCollection);

var _Method = require('./errors/Method');

var _Method2 = _interopRequireDefault(_Method);

var _Model = require('./base/Model');

var _Model2 = _interopRequireDefault(_Model);

var _Event = require('./base/Event');

var _Event2 = _interopRequireDefault(_Event);

var _isEvent = require('./isEvent');

var _isEvent2 = _interopRequireDefault(_isEvent);

var _events = require('./mixins/events');

var _events2 = _interopRequireDefault(_events);

var _bubbleUpEvent = require('./utils/bubbleUpEvent');

var _bubbleUpEvent2 = _interopRequireDefault(_bubbleUpEvent);

var _wrapCustomMethod = require('./utils/wrapCustomMethod');

var _wrapCustomMethod2 = _interopRequireDefault(_wrapCustomMethod);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function createModel() {
  var schema = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var methods = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var initializers = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

  var Model = function (_BaseModel) {
    _inherits(Model, _BaseModel);

    function Model() {
      var givenAttributes = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      _classCallCheck(this, Model);

      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Model).call(this, givenAttributes));

      var self = _this;

      var attributes = {};

      // others listening to this
      var listeners = {};

      // apply mixins
      (0, _events2.default)(_this, listeners);

      // built-in methods
      Object.defineProperty(_this, 'toJS', {
        value: function value() {
          function convertToJS(attrs) {
            return _lodash2.default.mapValues(attrs, function (v, k) {
              if ((0, _isModel2.default)(v) || (0, _isCollection2.default)(v)) {
                return v.toJS();
              }

              if (_lodash2.default.isPlainObject(v)) {
                return convertToJS(v);
              }

              return v;
            });
          }

          return convertToJS(attributes);
        }
      });

      Object.defineProperty(_this, 'destroy', {
        value: function value() {
          this.trigger('method:call', new _Event2.default({ path: ['destroy'] }));
          this.trigger('destroy');
          this.trigger('method:change', new _Event2.default({ path: ['destroy'] }));
          this.off();

          _lodash2.default.each(attributes, function (v, k) {
            if ((0, _isModel2.default)(v) || (0, _isCollection2.default)(v)) {
              v.destroy();
            }
          });
        }
      });

      Object.defineProperty(_this, 'getIn', {
        value: function value(paths) {
          if (!_lodash2.default.isArray(paths)) {
            throw new _Method2.default('`path` array is not provided');
          }

          var reducedPaths = [];
          return paths.reduce(function (result, path) {
            reducedPaths.push(path);

            if (!isNaN(path)) {
              // collection
              if (!(0, _isCollection2.default)(result)) {
                var collectionPath = _lodash2.default.take(reducedPaths, reducedPaths.length - 1);
                throw new _Method2.default('Path ' + JSON.stringify(collectionPath) + ' is not inside a collection');
              }

              return result.at(path);
            }

            // model
            if (!path in result) {
              throw new _Method2.default('Path ' + JSON.stringify(reducedPaths) + ' does not exist');
            }

            return result[path];
          }, this);
        }
      });

      // parse by schema
      var applySchema = _Types2.default.object.of(schema);
      attributes = applySchema(givenAttributes);

      // define attributes
      _lodash2.default.each(attributes, function (value, attributeName) {
        Object.defineProperty(_this, attributeName, {
          get: function get() {
            return attributes[attributeName];
          },
          set: function set(newValue) {
            try {
              schema[attributeName](newValue);
              attributes[attributeName] = newValue;

              self.trigger('change', new _Event2.default({
                path: [attributeName]
              }));
            } catch (typeError) {
              throw typeError;
            }
          },


          enumerable: true
        });

        // watch children
        if ((0, _isModel2.default)(value) || (0, _isCollection2.default)(value)) {
          (function () {
            var changeWatcher = (0, _bubbleUpEvent2.default)(self, value, 'change', [attributeName]);
            var methodCallWatcher = (0, _bubbleUpEvent2.default)(self, value, 'method:call', [attributeName]);
            var methodChangeWatcher = (0, _bubbleUpEvent2.default)(self, value, 'method:change', [attributeName]);

            value.on('destroy', function () {
              self.trigger('change', new _Event2.default({
                path: [attributeName]
              }));

              changeWatcher();
              methodCallWatcher();
              methodChangeWatcher();
            });
          })();
        }
      });

      // define methods
      _lodash2.default.each(methods, function (func, methodName) {
        if (typeof attributes[methodName] !== 'undefined' || typeof _this[methodName] !== 'undefined') {
          throw new _Method2.default('conflicting method name: ' + methodName);
        }

        _this[methodName] = (0, _wrapCustomMethod2.default)(_this, methodName, func);
      });

      // initializers
      initializers.forEach(function (initializer) {
        initializer(_this);
      });
      return _this;
    }

    return Model;
  }(_Model2.default);

  return Model;
}