'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = events;
function events(context) {
  var listeners = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  Object.defineProperty(context, 'on', {
    value: function value(event, fn) {
      if (typeof listeners[event] === 'undefined') {
        listeners[event] = [];
      }

      listeners[event].push(fn);

      return function cancelListener() {
        return context.off(event, fn);
      };
    }
  });

  Object.defineProperty(context, 'trigger', {
    value: function value(event) {
      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      if (typeof listeners[event] === 'undefined') {
        return;
      }

      return listeners[event].forEach(function (listener) {
        listener.apply(undefined, args);
      });
    }
  });

  Object.defineProperty(context, 'off', {
    value: function value() {
      var event = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];
      var fn = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

      if (!event) {
        listeners = {}; // eslint-disable-line

        return;
      }

      if (!fn) {
        listeners[event] = [];

        return;
      }

      if (typeof listeners[event] === 'undefined') {
        return;
      }

      listeners[event].forEach(function (listener, index) {
        if (listener === fn) {
          listeners[event].splice(index, 1);
        }
      });
    }
  });
}